"""Schedule plugin

Provides duty schedule / duty roster functions
"""
from dateutil import parser
from datetime import datetime, timedelta
from flask import Blueprint, request, jsonify
from werkzeug.exceptions import BadRequest, NotFound

from flaschengeist.database import db
from flaschengeist.plugins import Plugin
from flaschengeist.decorator import login_required

from . import event_controller, permissions
from .models import EventType

schedule_bp = Blueprint("schedule", __name__, url_prefix="/schedule")


class SchedulePlugin(Plugin):
    def __init__(self, config):
        super().__init__(
            blueprint=schedule_bp,
            permissions=permissions.permissions,
        )


@schedule_bp.route("/events/<int:event_id>", methods=["GET"])
@login_required()
def get_event(event_id, current_session):
    """Get event by id

    Route: ``/events/<event_id>`` | Method: ``GET``

    Args:
        event_id: ID identifying the event
        current_session: Session sent with Authorization Header

    Returns:
        JSON encoded event object
    """
    event = event_controller.get_event(event_id)
    if not event:
        raise NotFound
    return jsonify(event)


@schedule_bp.route("/events", methods=["GET"])
@schedule_bp.route("/events/<int:year>/<int:month>", methods=["GET"])
@schedule_bp.route("/events/<int:year>/<int:month>/<int:day>", methods=["GET"])
@login_required()
def __get_events(year=datetime.now().year, month=datetime.now().month, day=None, **kwargs):
    """Get Event objects for specified date (or month or year),
     if nothing set then events for current month are returned

    Args:
        year (int, optional): year to query, defaults to current year
        month (int, optional): month to query (if set), defaults to current month
        day (int, optional): day to query events for (if set)
        **kwargs: contains at least current_session (see flaschengeist.decorator)
    Returns:
        JSON list containing events found
    Raises:
        BadRequest: If date is invalid
    """
    try:
        begin = datetime(year=year, month=month, day=1)
        if day:
            begin += timedelta(days=day - 1)
            end = begin + timedelta(days=1)
        else:
            if month == 12:
                end = datetime(year=year + 1, month=1, day=1)
            else:
                end = datetime(year=year, month=month + 1, day=1)

        events = event_controller.get_events(begin, end)
        return jsonify(events)
    except ValueError:
        raise BadRequest("Invalid date given")


@schedule_bp.route("/event-types", methods=["POST"])
@login_required(permission=permissions.EVENT_TYPE)
def __new_event_kind(**kwargs):
    data = request.get_json()
    if "name" not in data:
        raise BadRequest
    kind = event_controller.create_event_kind(data["name"])
    return jsonify({"ok": "ok", "id": kind.id})


@schedule_bp.route("/slotKinds", methods=["POST"])
@login_required(permission=permissions.EVENT_TYPE)
def __new_slot_kind(**kwargs):
    data = request.get_json()
    if not data or "name" not in data:
        raise BadRequest
    kind = event_controller.create_job_kind(data["name"])
    return jsonify({"ok": "ok", "id": kind.id})


@schedule_bp.route("/events", methods=["POST"])
@login_required(permission=permissions.CREATE)
def __new_event(**kwargs):
    data = request.get_json()
    event = event_controller.create_event(
        begin=parser.isoparse(data["begin"]),
        end=parser.isoparse(data["end"]),
        description=data["description"],
        type=EventType.query.get(data["kind"]),
    )
    return jsonify({"ok": "ok", "id": event.id})


@schedule_bp.route("/events/<int:event_id>", methods=["DELETE"])
@login_required(permission=permissions.DELETE)
def __delete_event(event_id, **kwargs):
    if not event_controller.delete_event(event_id):
        raise NotFound
    db.session.commit()
    return jsonify({"ok": "ok"})


@schedule_bp.route("/eventKinds/<int:event_id>", methods=["PUT"])
@login_required(permission=permissions.EVENT_TYPE)
def __edit_event_kind(event_id, **kwargs):
    data = request.get_json()
    if not data or "name" not in data:
        raise BadRequest
    event_controller.rename_event_kind(event_id, data["name"])
    return jsonify({"ok": "ok"})


@schedule_bp.route("/events/<int:event_id>/slots", methods=["GET"])
@login_required()
def __get_slots(event_id, **kwargs):
    event = event_controller.get_event(event_id)
    if not event:
        raise NotFound
    return jsonify({event.slots})


@schedule_bp.route("/events/<int:event_id>/slots/<int:slot_id>", methods=["GET"])
@login_required()
def __get_slot(event_id, slot_id, **kwargs):
    slot = event_controller.get_event_slot(slot_id, event_id)
    if slot:
        return jsonify(slot)
    raise NotFound


@schedule_bp.route("/events/<int:event_id>/slots/<int:slot_id>", methods=["DELETE"])
@login_required(permission=permissions.DELETE)
def __delete_slot(event_id, slot_id, **kwargs):
    if event_controller.delete_event_slot(slot_id, event_id):
        return jsonify({"ok": "ok"})
    raise NotFound


@schedule_bp.route("/events/<int:event_id>/slots/<int:slot_id>", methods=["PUT"])
@login_required(permission=permissions.EDIT)
def __update_slot(event_id, slot_id, **kwargs):
    data = request.get_json()
    if not data:
        raise BadRequest

    for job in data["jobs"]:
        event_controller.add_job(job.kind, job.user)
    if event_controller.delete_event_slot(slot_id, event_id):
        return jsonify({"ok": "ok"})
    raise NotFound


@schedule_bp.route("/events/<int:event_id>/slots", methods=["POST"])
@login_required(permission=permissions.EDIT)
def __add_slot(event_id, **kwargs):
    event = event_controller.get_event(event_id)
    if not event:
        raise NotFound
    data = request.get_json()
    attr = {"job_slots": []}
    try:
        if "start" in data:
            attr["start"] = parser.isoparse(data["start"])
        if "end" in data:
            attr["end"] = parser.isoparse(data["end"])
        for job in data["jobs"]:
            attr["job_slots"].append({"needed_persons": job["needed_persons"], "kind": job["kind"]})
    except KeyError:
        raise BadRequest("Missing data in request")
    event_controller.add_slot(event, **attr)
    return jsonify({"ok": "ok"})


def __edit_event():
    ...
